@@ -15,8 +15,9 @@
 
 import logging
 
-from urwid import (Pile, Columns, Text, ListBox)
+from urwid import Text
 from subiquitycore.ui.buttons import done_btn, cancel_btn
+from subiquitycore.ui.container import Columns, ListBox, Pile
 from subiquitycore.ui.interactive import EmailEditor
 from subiquitycore.ui.utils import Padding, Color
 from subiquitycore.view import BaseView
@@ -44,19 +45,6 @@ log = logging.getLogger("console_conf.views.identity")
 +---------------------------------------------------+
 '''
 
-class SubmittingEmailEditor(EmailEditor):
-
-    def __init__(self, mainview):
-        self.mainview = mainview
-        super().__init__(caption="")
-
-    def keypress(self, size, key):
-        if key == 'enter':
-            self.mainview.done(None)
-            return None
-        else:
-            return super().keypress(size, key)
-
 
 class IdentityView(BaseView):
 
@@ -66,7 +54,7 @@ class IdentityView(BaseView):
         self.opts = opts
         self.loop = loop
         self.items = []
-        self.email = SubmittingEmailEditor(self)
+        self.email = EmailEditor()
         self.error = Text("", align="center")
         self.progress = Text("", align="center")
 
@@ -82,14 +70,19 @@ class IdentityView(BaseView):
         ]
         super().__init__(ListBox(body))
 
+    def keypress(self, size, key):
+        if key == 'enter':
+            self.done(None)
+            return None
+        else:
+            return super().keypress(size, key)
+
     def _build_model_inputs(self):
         sl = [
             Columns(
                 [
                     ("weight", 0.2, Text("Email address:", align="right")),
-                    ("weight", 0.3,
-                     Color.string_input(self.email,
-                                        focus_map="string_input focus"))
+                    ("weight", 0.3, Color.string_input(self.email)),
                 ],
                 dividechars=4
             ),
@@ -101,8 +94,8 @@ class IdentityView(BaseView):
         done = done_btn(on_press=self.done)
 
         buttons = [
-            Color.button(done, focus_map='button focus'),
-            Color.button(cancel, focus_map='button focus')
+            Color.button(done),
+            Color.button(cancel),
         ]
         return Pile(buttons)
 
